package com.dky.calculate;


import com.dky.utils.entity.SysDeviceHeatScene;
import com.dky.utils.result.MatchedDevice;

import java.util.*;
import java.util.stream.Collectors;

public class KitchenCookScheme {

    public static List<MatchedDevice> getCookDevices(Integer peopleNum, List<SysDeviceHeatScene> sysDeviceHeatSceneList) {
        // 根据用餐人数获取烹饪设备类型最少推荐使用人数
        Integer cookDeviceType = getCookDeviceType(peopleNum, sysDeviceHeatSceneList);

        List<SysDeviceHeatScene> collect = sysDeviceHeatSceneList.stream().filter(SysDeviceHeatScene -> Objects.equals(SysDeviceHeatScene.getRecomMinPeopleNum(), cookDeviceType)).collect(Collectors.toList());

        List<MatchedDevice> list = new ArrayList<>();

        collect.forEach((device) ->{
            list.add(new MatchedDevice(1, device));
        });

       return list;
    }

    public static Integer getCookDeviceType(Integer peopleNum, List<SysDeviceHeatScene> sysDeviceHeatSceneList) {
        // 使用流对sysDeviceHeatSceneList进行处理，首先收集到一个TreeSet中，TreeSet会根据SysDeviceHeatScene的recomMinPeopleNum属性进行排序，
        // 然后将TreeSet转换为ArrayList。这样做的目的是为了确保后续处理时集合中的元素是按照recomMinPeopleNum升序排列的。
        ArrayList<SysDeviceHeatScene> collect = sysDeviceHeatSceneList.stream().collect(
                Collectors.collectingAndThen(
                        Collectors.toCollection(() -> new TreeSet<>(Comparator.comparing(SysDeviceHeatScene::getRecomMinPeopleNum))),
                        ArrayList::new
                )
        );

        // 对collect列表中的所有元素的recomMinPeopleNum属性进行统计分析，获取最小值、最大值等统计信息。
        IntSummaryStatistics intSummaryStatistics = collect.stream().mapToInt(SysDeviceHeatScene::getRecomMinPeopleNum).summaryStatistics();

        // 遍历处理后的列表，找出满足条件的SysDeviceHeatScene的recomMinPeopleNum属性值。
        // 首先检查当前元素的recomMinPeopleNum是否大于等于列表中的最大值，如果是，则直接返回该值。
        // 如果不是，则判断当前人数是否在该场景建议的最小和最大人数范围内，如果是，则返回该场景建议的最小人数。
        for (SysDeviceHeatScene s : collect){
            if (s.getRecomMinPeopleNum() >= intSummaryStatistics.getMax()){
                return s.getRecomMinPeopleNum();
            } else {
                boolean f1 = peopleNum >= s.getRecomMinPeopleNum();
                boolean f2 = peopleNum <= s.getRecomMaxPeopleNum();
                if (f1 && f2){
                    return s.getRecomMinPeopleNum();
                }
            }
        }
        return 1;
    }




}
