package calculate;


import com.dky.utils.ConfigReader;
import com.dky.utils.entity.SysDeviceHeatScene;
import com.dky.utils.enums.DeviceSubType;
import com.dky.utils.enums.MetalKinTemps;
import com.dky.utils.result.MatchedDevice;

import java.util.*;
import java.util.stream.Collectors;


public class MetalKilnScheme {


    public static List<MatchedDevice> calSchemeByTechType(Double workArea, List<SysDeviceHeatScene> sysDeviceHeatSceneList) {
        // 对技术类型下的设备list按照单台设备工作容积进行排序
        sysDeviceHeatSceneList.sort((o1, o2) -> Double.compare(o2.getDevWorkArea(), o1.getDevWorkArea()));
        Double remainArea = workArea;
        // 遍历设备，根据工作容积进行匹配设备数量
        List<MatchedDevice> matchedDeviceList = new ArrayList<>();

        for (int i = 0; i < sysDeviceHeatSceneList.size(); i++) {
            SysDeviceHeatScene deviceHeatScene = sysDeviceHeatSceneList.get(i);
            double v = remainArea / deviceHeatScene.getDevWorkArea();
            int count = (int) (v);
            if (v < 1) {
                count = count + 1;
            }
            remainArea = remainArea - count * deviceHeatScene.getDevWorkArea();
            matchedDeviceList.add(new MatchedDevice(count, deviceHeatScene));
            if (remainArea <= 0) {
                // 当总供暖面积已经满足需求时，跳出循环，不再选择其他设备
                break;
            } else {
                if (i == sysDeviceHeatSceneList.size() - 1 && remainArea > 0) {
                    MatchedDevice device = matchedDeviceList.get(matchedDeviceList.size() - 1);
                    device.setCount(device.getCount() + 1);
                }
            }
        }
        return matchedDeviceList;
    }

    public static List<SysDeviceHeatScene> getSchemeList(String userIndustry, Double needTemp, List<SysDeviceHeatScene> sysDeviceHeatSceneList) {
        // 黑色金属矿采选业 行业编码
        String property = ConfigReader.getProperty("industry_code");
        Map<String, Object> map = new HashMap<>();
        List<SysDeviceHeatScene> sceneDevicesList = new ArrayList<>();

        // 将sysDeviceHeatSceneList流转换为排序后的ArrayList<SysDeviceHeatScene>对象。
        // 排序依据是SysDeviceHeatScene对象的getDevTechType方法返回的值。
        ArrayList<SysDeviceHeatScene> ary = sysDeviceHeatSceneList.stream().collect(
                Collectors.collectingAndThen(
                        // 使用TreeSet收集元素，以保证收集后的集合是按照getDevTechType方法返回值排序的。
                        Collectors.toCollection(() -> new TreeSet<>(Comparator.comparing(SysDeviceHeatScene::getDevTechType))),
                        // 将TreeSet转换为ArrayList。
                        ArrayList::new
                )
        );

        // 遍历ary数组中的每个元素
        ary.forEach((item) -> {
            List<SysDeviceHeatScene> list = new ArrayList<>();
            // 遍历sysDeviceHeatSceneList列表，筛选出与当前item的设备技术类型相同的设备
            sysDeviceHeatSceneList.forEach((device) -> {
                if (device.getDevTechType().equals(item.getDevTechType())) {
                    list.add(device); // 将符合条件的设备添加到list中
                }
            });
            // 将按照设备技术类型分组后的设备列表，添加到map中
            map.put(item.getDevTechType(), list);
        });

        if (userIndustry.equals(property)) {
            sceneDevicesList = (List<SysDeviceHeatScene>) map.get(DeviceSubType.Metal_Kiln_Electric_Arc.getDesc());
        } else {
            if (needTemp >= MetalKinTemps.Metal_Kiln_Direct_Heat_Min_Temp.getTemp() && needTemp <= MetalKinTemps.Metal_Kiln_Direct_Heat_Max_Temp.getTemp()){
                sceneDevicesList = (List<SysDeviceHeatScene>) map.get(DeviceSubType.Metal_Kiln_Direct_Heat.getDesc());
            } else if (needTemp >= MetalKinTemps.Metal_Kiln_Interstitial_Heat_Min_Temp.getTemp() && needTemp <= MetalKinTemps.Metal_Kiln_Interstitial_Heat_Max_Temp.getTemp()){
                sceneDevicesList = (List<SysDeviceHeatScene>) map.get(DeviceSubType.Metal_Kiln_Interstitial_Heat.getDesc());
            } else if (needTemp >= MetalKinTemps.Metal_Kiln_Induction_Min_Temp.getTemp() && needTemp <= MetalKinTemps.Metal_Kiln_Induction_Max_Temp.getTemp()) {
                sceneDevicesList = (List<SysDeviceHeatScene>) map.get(DeviceSubType.Metal_Kiln_Induction.getDesc());
            }
        }

        return sceneDevicesList;

    }

    public static void main(String[] args) {
        String property = ConfigReader.getProperty("industry_code");
        System.out.println(property);
    }

}
