package com.dky.stirtpar;


import cn.hutool.json.JSONObject;
import com.dky.modelI.DntdModelI;
import com.dky.utils.CalculateUtils;
import com.dky.utils.entity.SysDeviceHeatScene;

import java.io.BufferedReader;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.nio.charset.StandardCharsets;
import java.text.DecimalFormat;
import java.util.List;

public class StirparModelCalculate implements DntdModelI {

    DecimalFormat df = new DecimalFormat("#.00");

    /**
     * 计算热源消耗总量
     * @param provinceCode 省份编码
     * @param peopleNum 人口
     * @param averageGdp 人均GDP
     * @param U 城市化率
     * @param I 工业增加值
     * @param II 工业能源强度
     * @return
     */
    public static Double energyConsumeStirCoefficient(String provinceCode, Integer peopleNum, Double averageGdp, Double U, Double I, Double II) {
        JSONObject jo = (JSONObject) toJSON("energyConsumeStirpatCoefficient.json").get(provinceCode);
        Double a = Double.parseDouble(jo.get("a").toString());
        Double b1 = Double.parseDouble(jo.get("b1").toString());
        Double b2 = Double.parseDouble(jo.get("b2").toString());
        Double b3 = Double.parseDouble(jo.get("b3").toString());
        Double b4 = Double.parseDouble(jo.get("b4").toString());
        Double b5 = Double.parseDouble(jo.get("b5").toString());
        Double b6 = Double.parseDouble(jo.get("b6").toString());
        Double e = Double.parseDouble(jo.get("e").toString());

        Double add1 = CalculateUtils.add(a, b1 * Math.log(peopleNum));
        Double add2 = CalculateUtils.add(add1, b2 * Math.log(U));
        Double add3 = CalculateUtils.add(add2, b3 * Math.log(averageGdp));
        Double add4 = CalculateUtils.add(add3, b4 * Math.pow(Math.log(averageGdp), 2));
        Double add5 = CalculateUtils.add(add4, b5 * Math.log(I));
        Double add6 = CalculateUtils.add(add5, b6 * Math.log(II));
        Double add7 = CalculateUtils.add(add6, e);

        return Math.round(Math.exp(add7) * 100) / 100.0;
    }

    /**
     * 计算电耗消耗总量
     * @param provinceCode 省份编码
     * @param CT 能源消耗量
     * @param peopleNum 人口
     * @param averageGdp 人均GDP
     * @param U 城市化率
     * @param PR 电力价格
     * @param T 年均温度
     * @return
     */
    public static Double electricConsumeStirCoefficient(String provinceCode, Double CT, Integer peopleNum, Double averageGdp, Double U, Double PR, Double T) {
        JSONObject jo = (JSONObject) toJSON("electricConsumeStirpatCoefficient.json").get(provinceCode);
        Double a = Double.parseDouble(jo.get("a").toString());
        Double b1 = Double.parseDouble(jo.get("b1").toString());
        Double b2 = Double.parseDouble(jo.get("b2").toString());
        Double b3 = Double.parseDouble(jo.get("b3").toString());
        Double b4 = Double.parseDouble(jo.get("b4").toString());
        Double b5 = Double.parseDouble(jo.get("b5").toString());
        Double b6 = Double.parseDouble(jo.get("b6").toString());
        Double b7 = Double.parseDouble(jo.get("b7").toString());
        Double e = Double.parseDouble(jo.get("e").toString());

        Double add1 = CalculateUtils.add(a, b1 * Math.log(CT));
        Double add2 = CalculateUtils.add(add1, b2 * Math.log(peopleNum));
        Double add3 = CalculateUtils.add(add2, b3 * Math.log(U));
        Double add4 = CalculateUtils.add(add3, b4 * Math.log(U));
        Double add5 = CalculateUtils.add(add4, b5 * Math.log(averageGdp));
        Double add6 = CalculateUtils.add(add5, b6 * Math.log(PR));
        Double add7 = CalculateUtils.add(add6, b7 * Math.log(T));
        Double add8 = CalculateUtils.add(add7, e);

        return Math.round(Math.exp(add8) * 100) / 100.0;
    }



    public static JSONObject toJSON(String fileName) {
        // 使用ClassLoader的getResourceAsStream方法读取资源文件
        InputStream inputStream = StirparModelCalculate.class.getClassLoader().getResourceAsStream(fileName);
        if (inputStream == null) {
            System.out.println("读取文件失败");
        }
        // 使用BufferedReader读取InputStream中的内容
        BufferedReader reader = new BufferedReader(new InputStreamReader(inputStream, StandardCharsets.UTF_8));
        StringBuilder stringBuilder = new StringBuilder();
        String currentLine;
        try {
            while ((currentLine = reader.readLine()) != null) {
                stringBuilder.append(currentLine);
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        // 关闭BufferedReader
        try {
            reader.close();
        } catch (Exception e) {
            e.printStackTrace();
        }

        return new JSONObject(stringBuilder.toString());
    }



    @Override
    public JSONObject createReport(JSONObject jsonObject, List<SysDeviceHeatScene> alternateDeviceList) {
        JSONObject electricInfo = new JSONObject();
        JSONObject energyInfo = new JSONObject();

        try {
            energyInfo = (JSONObject) jsonObject.get("energyInfo");
            electricInfo = (JSONObject) jsonObject.get("electricInfo");
        } catch (Exception e) {
            e.printStackTrace();
        }

        String provinceCode = jsonObject.get("provinceCode").toString();
        Integer peopleNum = Integer.parseInt(new DecimalFormat("#").format(energyInfo.get("peopleNum")));
        Double averageGdp = Double.parseDouble(energyInfo.get("averageGdp").toString());
        Double U = Double.parseDouble(energyInfo.get("ut").toString());
        Double I = Double.parseDouble(energyInfo.get("i1").toString());
        Double II = Double.parseDouble(energyInfo.get("i2").toString());
        Double PR = Double.parseDouble(electricInfo.get("PR").toString());
        Double CT = Double.parseDouble(electricInfo.get("CT").toString());
        Double T = Double.parseDouble(electricInfo.get("T").toString());

        JSONObject resultJsonObject = new JSONObject();
        resultJsonObject.put("CT", df.format(energyConsumeStirCoefficient(provinceCode, peopleNum, averageGdp, U, I, II)));
        resultJsonObject.put("CE", df.format(electricConsumeStirCoefficient(provinceCode, CT, peopleNum, averageGdp, U, PR, T)));

        return resultJsonObject;
    }
}
